@@ -3,19 +3,14 @@ from __future__ import unicode_literals
 
 from django.core.cache import cache
 from django.db import models
-from django.db.models.signals import post_save, post_delete, pre_delete
-from django.dispatch import receiver
-try:
-    from django.utils.six import PY3
-except ImportError:
-    # Django < 1.5. No Python 3 support
-    PY3 = False
+from django.utils.encoding import python_2_unicode_compatible
 
-from .conf import PLACEHOLDER_TEMPLATE
 from .conf import CACHE_TIMEOUT, SECTION_CACHE_KEY, PLACEMENTS_KEY_FORMAT
+from .conf import PLACEHOLDER_TEMPLATE
 from .validators import validate_pattern
 
 
+@python_2_unicode_compatible
 class Section(models.Model):
     "A grouping of site urls."
 
@@ -27,10 +22,6 @@ class Section(models.Model):
     def __str__(self):
         return self.name
 
-    if not PY3:
-        __unicode__ = __str__
-        __str__ = lambda self: self.__unicode__().encode('utf-8')
-
 
 def retrieve_all_sections():
     "Get all sections from the cache or query the database."
@@ -43,15 +34,7 @@ def retrieve_all_sections():
     return sections
 
 
-@receiver(post_save, sender=Section)
-@receiver(post_delete, sender=Section)
-def cycle_sections_cache(sender, **kwargs):
-    "Delete and restore section info in the cache."
-    cache.delete(SECTION_CACHE_KEY)
-    if CACHE_TIMEOUT:
-        retrieve_all_sections()
-
-
+@python_2_unicode_compatible
 class Size(models.Model):
     "Common Ad size."
 
@@ -60,13 +43,10 @@ class Size(models.Model):
     height = models.PositiveSmallIntegerField()
 
     def __str__(self):
-        return '{0} {1}x{2}'.format(self.name, self.width, self.height)
-
-    if not PY3:
-        __unicode__ = __str__
-        __str__ = lambda self: self.__unicode__().encode('utf-8')
+        return '{name} {width}x{height}'.format(name=self.name, width=self.width, height=self.height)
 
 
+@python_2_unicode_compatible
 class Placement(models.Model):
     "Ad to be rendered in given sections."
 
@@ -77,11 +57,7 @@ class Placement(models.Model):
     sections = models.ManyToManyField(Section, blank=True, related_name='placements')
 
     def __str__(self):
-        return '{0} ({1})'.format(self.name, self.size)
-
-    if not PY3:
-        __unicode__ = __str__
-        __str__ = lambda self: self.__unicode__().encode('utf-8')
+        return '{} ({})'.format(self.name, self.size)
 
     @property
     def placeholder(self):
@@ -108,30 +84,9 @@ def retrieve_section_placements(section):
             cache.set(cache_key, placements, CACHE_TIMEOUT)
     return placements
 
-
-def _update_placement_cache(placement, replace=True):
-    "Remove placement from related section caches. Replace if requested."
-    if CACHE_TIMEOUT:
-        for section in placement.sections.all():
-            cache_key = PLACEMENTS_KEY_FORMAT.format(section.pk)
-            placements = cache.get(cache_key, [])
-            try:
-                placements.remove(placement)
-            except ValueError:
-                # Placement not in the list
-                pass
-            if replace:
-                placements.append(placement)
-            cache.set(cache_key, placements, CACHE_TIMEOUT)
-
-
-@receiver(post_save, sender=Placement)
-def save_placement_handler(sender, instance,  **kwargs):
-    "Add or update the placement in the caches."
-    _update_placement_cache(placement=instance, replace=True)
-
-
-@receiver(pre_delete, sender=Placement)
-def delete_placement_handler(sender, instance,  **kwargs):
-    "Remove the placement from the section caches."
-    _update_placement_cache(placement=instance, replace=False)
+try:
+    from django import apps
+except ImportError:
+    # Django < 1.7
+    # Manually import the signal recivers.
+    from . import recievers
